## pluginhandler for jabberif
##

from string import split,strip,join
import sys,os
import imp			## module loading
import traceback		## module import + function execution
import jabber

class pluginhandler:
	def __init__(self, uh, lh, config):
		self.__plugins = {}
		self.__commands = {}
		self.__uh = uh ## userhandler instance
		self.__lh = lh ## loghandler instance
		self.__config = config
	
	def plugins(self):
		return self.__plugins
	def commands(self):
		return self.__commands

	def load(self, name):
		"""load plugin module"""
		if self.__plugins.has_key(name):
			self.__lh.log("error loading: plugin %s already exists" % name, 'error')
			return 1
			
		if not os.path.isfile(self.__config['plugindir']+'/'+name+'.py'):
			self.__lh.log("error loading: plugin %s doesn't exist" % name, 'error')
			return 1

		fp, pathname, description = imp.find_module(name, [self.__config['plugindir']])
		try:
			module = imp.load_module(name, fp, pathname, description)
			self.__plugins[name]={'name': name, 'module': module,
				'obj': eval('module.'+name+'(self)')}
		except ImportError, e:
			self.__lh.log("error importing module %s: %s" % (name, str(e)), 'error')
			if fp:
				fp.close()
			return 1
		except Exception, e:
			self.unload(name)
			self.__lh.log("error loading module %s: %s" % (name, str(e)), 'error')
			self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
			if fp:
				fp.close()
			return 1
		if fp:
			fp.close()
		return 0
	
	def register_command(self, module, name, function, help='', args = {}):
		"""register command
			this is usually called by modules to register their new commands
			args can be:
				op_only: 0/1"""
		if self.__commands.has_key(name):
			self.__lh.log("error registering cmd: %s already exists in module %s" % (name, self.__commands[name]['module'].name), 'error')
		else:
			self.__lh.log("registering command /%s from module %s" % (name, module.name), 'module')
			self.__commands[name] = {'name': name, 'module': module, 'function': function,
				'help': help, 'args': args}
				
	def unload(self, name):
		"""unload plugin module"""
		if not self.__plugins.has_key(name):
			self.__lh.log("error unloading: plugin %s is not loaded" % name, 'error')
			return

		module = self.__plugins[name]['obj']

		if hasattr(module, '_unload_') and callable(module._unload_):
			try:
				module._unload_()
			except Exception, e:
				self.__lh.log("error calling _unload_ in module %s: %s" % (name, str(e)), 'error')
				self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
				

		for cmd in self.__commands.keys():
			if self.__commands[cmd]['module'] is module:
				self.__commands.pop(cmd)
				
		self.__plugins.pop(name)
	
	def unload_all(self):
		for module in self.__plugins.keys():
			self.unload(module)

	def send(self, con, to, message):
		"""send message"""
		msg = jabber.Message(to, message)
		msg.setType('chat')
		con.send(msg)
		self.__lh.log(str(to) + ': ' + message, 'outmessage')
		
	def send_back(self, con, msg, message):
		"""send message back to where it came from"""
		self.send(con, msg.getFrom(), message)
	
	def commandhandler(self, args):
		"""here the commands are relayed to modules"""
		con = args['con']
		msg = args['msg']
		cmd = args['cmd']

		if len(cmd) <= 0:
			return

		if self.__commands.has_key(cmd[0]):
			command = self.__commands[cmd[0]]
			if command['args'].has_key('op_only') and command['args']['op_only'] == 1 and not self.__uh.check_op(msg.getFrom().getStripped()):
				self.send_back(con, msg, 'sorry. you require op status to execute this command')
			else:
				try:
					command['function'](args)
				except Exception, e:
					self.__lh.log("error executing command %s: %s" % (cmd[0], str(e)), 'error')
					self.__lh.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
					self.send_back(con, msg, 'error executing function')
				
		else:
			self.send_back(con, msg, 'error. try /help')

	def get_config(self, key):
		"""get config file entry from section 'plugins'"""
		if self.__config.has_key(key):
			return self.__config[key]
		return None

	def get_userhandler(self):
		"""return userhandler"""
		return self.__uh
		
	def get_loghandler(self):
		"""return loghandler"""
		return self.__lh

